import random
import uuid
from urllib.parse import urlparse

from django.apps import apps as django_apps
from django.contrib.auth import authenticate, get_user_model, login
from django.core.exceptions import ImproperlyConfigured
from django.shortcuts import resolve_url

from . import settings


def maybe_create_guest_user(request):
    """
    Create a guest user and log them in.

    This function will create and authenticate a new guest user should the visitor
    not be authenticated already and their user agent isn't on the block list.

    """
    assert hasattr(
        request, "session"
    ), "Please add 'django.contrib.sessions' to INSTALLED_APPS."

    if settings.ENABLED and request.user.is_anonymous:
        user_agent = request.META.get("HTTP_USER_AGENT", "")
        if not settings.BLOCKED_USER_AGENTS.search(user_agent):
            UserModel = get_user_model()
            Guest = get_guest_model()
            user = Guest.objects.create_guest_user(request)
            user = authenticate(
                request=request,
                username=getattr(user, UserModel.USERNAME_FIELD),
            )
            assert user, (
                "Guest authentication failed. Do you have "
                "'guest_user.backends.GuestBackend' in AUTHENTICATION_BACKENDS?"
            )
            login(request, user)


def get_guest_model():
    """
    Return the configured Guest model.
    """
    try:
        return django_apps.get_model(settings.MODEL, require_ready=False)
    except ValueError:
        raise ImproperlyConfigured(
            "GUEST_USER_MODEL must be of the form 'app_label.model_name'"
        )
    except LookupError:
        raise ImproperlyConfigured(
            "GUEST_USER_MODEL refers to model '%s' that has not been installed"
            % settings.MODEL
        )


def is_guest_user(user) -> bool:
    """
    Check if the given user instance is a temporary guest.

    """

    if user.is_anonymous:
        return False

    if getattr(user, "backend", None) == "guest_user.backends.GuestBackend":
        return True

    GuestModel = get_guest_model()
    return GuestModel.objects.filter(user=user).exists()


def generate_uuid_username() -> str:
    """Generate a random username based on UUID."""
    UserModel = get_user_model()
    max_length = UserModel._meta.get_field(UserModel.USERNAME_FIELD).max_length
    return uuid.uuid4().hex[:max_length]


def generate_numbered_username() -> str:
    """Generate a random username based on a prefix and a random number."""
    prefix = settings.NAME_PREFIX
    digits = settings.NAME_SUFFIX_DIGITS
    number = random.randint(1, (10**digits) - 1)
    return f"{prefix}{number:{f'0{digits}'}}"


def generate_friendly_username() -> str:
    """
    Generate a random username with adjective and nouns put together.

    Requires `random-username` to be installed.
    """
    from random_username.generate import generate_username

    return generate_username()[0]


def redirect_with_next(request, redirect_url, redirect_field_name):
    """
    Redirect the user to a login page with a "next" parameter.

    :meta private:

    """
    # https://github.com/django/django/blob/ba9ced3e9a643a05bc521f0a2e6d02e3569de374/django/contrib/auth/decorators.py#L22-L33  # noqa

    path = request.build_absolute_uri()
    resolved_login_url = resolve_url(redirect_url)

    # If the login url is the same scheme and net location then just
    # use the path as the "next" url.
    login_scheme, login_netloc = urlparse(resolved_login_url)[:2]
    current_scheme, current_netloc = urlparse(path)[:2]
    if (not login_scheme or login_scheme == current_scheme) and (
        not login_netloc or login_netloc == current_netloc
    ):
        path = request.get_full_path()
    from django.contrib.auth.views import redirect_to_login

    return redirect_to_login(path, resolved_login_url, redirect_field_name)



from django.conf import settings as django_settings
from django.contrib.auth import REDIRECT_FIELD_NAME, authenticate, get_user_model, login
from django.shortcuts import redirect, resolve_url
from django.utils.http import url_has_allowed_host_and_scheme
from django.utils.module_loading import import_string
from django.views.generic import FormView, TemplateView

from . import settings
from .exceptions import NotGuestError
from .functions import get_guest_model, is_guest_user


class ConvertFormView(FormView):
    """
    Allow a guest to convert to a regular user.

    """

    anonymous_redirect = None
    user_redirect = None

    success_url = None
    redirect_field_name = REDIRECT_FIELD_NAME
    template_name = "guest_user/convert_form.html"

    success_url_allowed_hosts = set()

    def dispatch(self, request, *args, **kwargs):
        if request.user.is_anonymous:
            return redirect(self.get_anonymous_redirect())

        if not is_guest_user(request.user):
            return redirect(self.get_user_redirect())

        return super().dispatch(request, *args, **kwargs)

    def get_anonymous_redirect(self):
        """Return the URL to redirect anonymous users to."""
        return self.anonymous_redirect or django_settings.LOGIN_URL

    def get_user_redirect(self):
        """Return the URL to redirect regular users to."""
        return self.user_redirect or django_settings.LOGIN_REDIRECT_URL

    def get_form_class(self):
        return self.form_class or import_string(settings.CONVERT_FORM)

    def get_initial(self):
        """Return the initial data to use for forms on this view."""
        initial = super().get_initial()
        if settings.CONVERT_PREFILL_USERNAME:
            UserModel = get_user_model()
            initial["username"] = getattr(self.request.user, UserModel.USERNAME_FIELD)
        return initial

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        if self.request.method == "POST":
            kwargs["instance"] = self.request.user
        return kwargs

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({self.redirect_field_name: self.get_redirect_url()})
        return context

    def form_valid(self, form):
        """Security check complete. Log the user in."""
        GuestModel = get_guest_model()

        try:
            GuestModel.objects.convert(form)
        except NotGuestError:
            # Redirect if it's already a regular user.
            pass
        else:
            # Authenticate the user with standard backend.
            login(self.request, authenticate(self.request, **form.get_credentials()))

        return redirect(self.get_success_url())

    def get_success_url(self):
        return self.get_redirect_url() or self.get_default_redirect_url()

    def get_redirect_url(self):
        """Return the user-originating redirect URL if it's safe."""
        redirect_to = self.request.POST.get(
            self.redirect_field_name, self.request.GET.get(self.redirect_field_name, "")
        )
        url_is_safe = url_has_allowed_host_and_scheme(
            url=redirect_to,
            allowed_hosts=self.get_success_url_allowed_hosts(),
            require_https=self.request.is_secure(),
        )
        return redirect_to if url_is_safe else ""

    def get_success_url_allowed_hosts(self):
        return {self.request.get_host(), *self.success_url_allowed_hosts}

    def get_default_redirect_url(self):
        """Return the default redirect URL."""
        return resolve_url(self.success_url or settings.CONVERT_REDIRECT_URL)


convert_form = ConvertFormView.as_view()


class ConvertSuccessView(TemplateView):
    template_name = "guest_user/convert_success.html"


convert_success = ConvertSuccessView.as_view()



from datetime import timedelta

from django.contrib import admin

from . import settings
from .functions import get_guest_model
from .models import Guest


class GuestAdmin(admin.ModelAdmin):
    list_display = ["user", "created_at", "is_expired"]
    actions = ["delete_expired_guests"]
    fields = ["user", "created_at"]
    readonly_fields = ["user", "created_at", "is_expired"]

    def is_expired(self, obj):
        return obj.is_expired()

    is_expired.boolean = True

    def delete_expired_guests(self, request, queryset):
        expired_guests = queryset.filter_expired()
        count = expired_guests.count()

        for guest in expired_guests:
            # Call delete method to trigger signals and cascades
            guest.user.delete()

        self.message_user(request, f"Deleted {count} guests.")

    delete_expired_guests.short_description = (
        "Delete selected guests older than {}".format(
            timedelta(seconds=settings.MAX_AGE)
        )
    )

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def get_deleted_objects(self, objs, request):
        """
        Show the correct CASCADE for guest deletion.

        This Guest admin's DELETE actions will always try to delete the associated
        user objects instead of the guest instances, to allow a full cascade.

        """
        user_objs = [obj.user for obj in objs]
        return super().get_deleted_objects(user_objs, request)

    def delete_model(self, request, obj):
        """
        Make the delete action cascade.

        """
        obj.user.delete()

    def delete_queryset(self, request, queryset):
        """
        Make the delete action cascade.

        """
        for obj in queryset:
            obj.user.delete()


if get_guest_model() == Guest:
    admin.site.register(Guest, GuestAdmin)



"""
The ``django-guest-user`` template tags are registered in the ``guest_user`` namespace.

To use them, load them in your template:

.. code:: jinja

    {% load guest_user %}

"""


from django.template import Library

from ..functions import is_guest_user as is_guest_user_func

register = Library()

is_guest_user = register.filter(is_guest_user_func)
"""
Template filter to check if the passed object is a guest user.

Usage

.. code:: jinja

  {% if user|is_guest_user %}
    Hello guest.
  {% endif %}

"""



from tos.middleware import UserAgreementMiddleware
from guest_user.functions import is_guest_user


class GuestUserAgreementMiddleware(UserAgreementMiddleware):
    def should_fast_skip(self, request):
        if super().should_fast_skip(request):
            return True
        return is_guest_user(request.user)



from functools import wraps

from django.conf import settings as django_settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.shortcuts import redirect

from . import settings
from .functions import is_guest_user, maybe_create_guest_user, redirect_with_next


def allow_guest_user(function=None):
    """
    Allow anonymous users to access the view by creating a guest user.

    Usage example::

        from guest_user.decorators import allow_guest_user

        @allow_guest_user
        def hello_world(request):
            return HttpResponse(f"Hello {request.user.username}!")

    """

    def decorator(view_func):
        @wraps(view_func)
        def wrapper(request, *args, **kwargs):
            maybe_create_guest_user(request)
            return view_func(request, *args, **kwargs)

        return wrapper

    if function:
        return decorator(function)
    return decorator


def guest_user_required(
    function=None,
    anonymous_url=None,
    registered_url=None,
):
    """
    Current user must be a temporary guest.

    Since being a guest user is not a state that a registered
    user can ever revert back to, there is no "next" URL handling
    in this decorator.

    :param anonymous_url: Redirect target for anonymous users.
      Defaults to :attr:`GUEST_USER_REQUIRED_ANON_URL<guest_user.app_settings.AppSettings.REQUIRED_ANON_URL>`.
    :param registered_url: Redirect target for registered users.
      Defaults to :attr:`GUEST_USER_REQUIRED_USER_URL<guest_user.app_settings.AppSettings.REQUIRED_USER_URL>`.

    Usage example::

        from guest_user.decorators import guest_user_required

        @guest_user_required(anonymous_url="/login/", registered_url="/dashboard/")
        def only_for_guests(request):
            pass

    """

    def decorator(view_func):
        @wraps(view_func)
        def wrapper(request, *args, **kwargs):
            if is_guest_user(request.user):
                return view_func(request, *args, **kwargs)
            if request.user.is_anonymous:
                redirect_url = anonymous_url or settings.REQUIRED_ANON_URL
            else:
                redirect_url = registered_url or settings.REQUIRED_USER_URL
            return redirect(redirect_url)

        return wrapper

    if function:
        return decorator(function)
    return decorator


def regular_user_required(
    function=None,
    login_url=None,
    convert_url=None,
    redirect_field_name=REDIRECT_FIELD_NAME,
):
    """
    Current user must not be a temporary guest.

    The redirected URL will get a "next" parameter added to the URL
    in order to redirect the user back to the page they were trying to access.

    :param login_url: Redirect target for anonymous users.
       Defaults to :ref:`django:ref/settings:``login_url```.
    :param convert_url: Redirect target for guest users.
      Defaults to :attr:`GUEST_USER_CONVERT_URL<guest_user.app_settings.AppSettings.CONVERT_URL>`.
    :param redirect_field_name: URL parameter used to redirect to the origin page. Defaults to
      :attr:`django.contrib.auth.REDIRECT_FIELD_NAME<django:django.contrib.auth.mixins.AccessMixin.redirect_field_name>`
      (= "next").

    Usage example::

        from guest_user.decorators import regular_user_required

        @regular_user_required
        def permanent_users_only(request):
            # This view will redirect anonymous users to the login page
            # and guest users to the convert page, respectively.
            # A redirect URL parameter will be added to users can jump right back here.
            pass

    """

    def decorator(view_func):
        @wraps(view_func)
        def wrapper(request, *args, **kwargs):
            user = request.user
            if user.is_authenticated and not is_guest_user(user):
                return view_func(request, *args, **kwargs)
            if user.is_anonymous:
                redirect_url = login_url or django_settings.LOGIN_URL
            else:
                redirect_url = convert_url or settings.CONVERT_URL
            return redirect_with_next(request, redirect_url, redirect_field_name)

        return wrapper

    if function:
        return decorator(function)
    return decorator



from django.core.exceptions import ValidationError
from django.dispatch import Signal, receiver

from allauth.socialaccount.signals import social_account_added

from ...functions import get_guest_model, is_guest_user


@receiver(social_account_added)
def convert_guest_with_social_login(sender, request, sociallogin, **kwargs):
    """
    Check if a guest user has added a social account.

    When a user logged in using a social account, the guest is converted.

    We will also ensure that the social account's username is used.

    """
    user = request.user

    if is_guest_user(user):
        # Convert the user right away, since the social account
        # has already been connected at this point.
        get_guest_model().objects.filter(user=user).delete()

        from allauth.account.adapter import get_adapter as get_account_adapter
        from allauth.socialaccount.adapter import get_adapter as get_social_adapter

        # Normally, allauth will only populate a user that registers using a social account,
        # but in this case the user already exists, so we need to populate it ourselves.
        social_adapter = get_social_adapter()
        social_adapter.populate_user(
            request,
            sociallogin,
            sociallogin.account.get_provider().extract_common_fields(
                sociallogin.account.extra_data
            ),
        )
        account_adapter = get_account_adapter()
        try:
            username = getattr(user, user.USERNAME_FIELD)
            account_adapter.clean_username(username)
        except ValidationError:
            # Empty the invalid username to allow fallbacks in `populate_username`
            setattr(user, user.USERNAME_FIELD, "")
        account_adapter.populate_username(request, user)
        user.save()

        converted_social_account.send(sender=sender, user=user, sociallogin=sociallogin)


converted_social_account = Signal()
"""
A guest user converted by connecting a social account.

:param user: The converted user object.
:param sociallogin: The social login object that converted the guest.

"""



from django.contrib.auth.forms import UserCreationForm as BaseUserCreationForm


class UserCreationForm(BaseUserCreationForm):
    """
    A modelform that creates a standard Django user.

    Custom implementations must implement :meth:`get_credentials`.

    """

    def get_credentials(self) -> dict:
        """
        Get the credentials required to log the user in after conversion.

        The credentials are passed to Django's :func:`authenticate()<django.contrib.auth.authenticate>`.

        :return: Login credentials. This is usually a dict with "username" and "password".

        """
        return {
            "username": self.cleaned_data["username"],
            "password": self.cleaned_data["password1"],
        }



from django.urls import path

from .views import convert_form, convert_success

urlpatterns = [
    path("", convert_form, name="guest_user_convert"),
    path("success/", convert_success, name="guest_user_convert_success"),
]



from django.dispatch import Signal

guest_created = Signal()
"""
A visitor accessed a view that created a guest user.

:param user: The new guest user.
:param request: The request that created the guest user.

"""

converted = Signal()
"""
A guest user converted to a regular registered user.

:param user: The now registered user.

"""



from datetime import timedelta

from django.conf import settings as django_settings
from django.contrib.auth import get_user_model
from django.db import IntegrityError, models, transaction
from django.forms import ModelForm
from django.utils.module_loading import import_string
from django.utils.timezone import now

from . import settings
from .exceptions import NotGuestError
from .functions import is_guest_user
from .signals import converted, guest_created

UserModel = get_user_model()


class GuestQuerySet(models.QuerySet):
    def filter_expired(self):
        delete_before = now() - timedelta(seconds=settings.MAX_AGE)
        return self.filter(
            created_at__lt=delete_before,
        ).select_related("user")


class GuestManager(models.Manager.from_queryset(GuestQuerySet)):
    """
    Manager for Guest objects.

    """

    @property
    def generate_username(self):
        return import_string(settings.NAME_GENERATOR)

    def create_guest_user(self, request=None, username: str = None) -> UserModel:
        """
        Create a guest user.

        Returns the underlying User object.

        :param request: The current request object.
        :param username: The preferred username for the user, may be None.

        """
        if username is None:
            username = self.generate_username()

        user = None
        while user is None:
            try:
                with transaction.atomic():
                    user = UserModel.objects.create_user(username, "")
            except IntegrityError:
                # retry with a new username
                username = self.generate_username()

        self.create(user=user)
        if request is not None:
            guest_created.send(self, user=user, request=request)
        return user

    def convert(self, form: ModelForm) -> UserModel:
        """
        Convert a guest user to a regular one.

        The form passed in is expected to be a ModelForm instance,
        bound to the user to be converted.

        :raises: TypeError if the user is not a temporary guest.
        :param form: The model form used to create the permanent user.
        :returns: The converted ``User`` object.

        """
        if not is_guest_user(form.instance):
            raise NotGuestError("You cannot convert a non guest user")

        user = form.save()

        # We need to remove the Guest instance assocated with the
        # newly-converted user
        self.filter(user=user).delete()
        converted.send(self, user=user)
        return user

    def delete_expired(self):
        """
        Delete all expired guest users.

        """
        for guest in self.filter_expired():
            # Call delete method to trigger signals and cascades
            guest.user.delete()


class Guest(models.Model):
    """
    A temporary guest user.

    Users linked to a Guest instance are considered temporary guests and will
    be deleted by cleanup jobs after their expiration.

    The age of a guest user is determined by the ``created_at`` field.

    This model is swappable with the :attr:`GUEST_USER_MODEL<guest_user.app_settings.AppSettings.MODEL>` setting.
    Custom Guest models should use the GuestManager or a custom manager that
    implements the same custom methods.

    """

    user = models.OneToOneField(
        to=django_settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        verbose_name="User",
        related_name="guest+",
    )

    created_at = models.DateTimeField(
        verbose_name="Created at",
        auto_now_add=True,
        db_index=True,
    )

    objects = GuestManager()

    class Meta:
        verbose_name = "Guest"
        verbose_name_plural = "Guests"
        swappable = "GUEST_USER_MODEL"
        ordering = ["-created_at"]

    def __str__(self):
        return str(self.user)

    def is_expired(self) -> bool:
        """
        Check if the guest user has expired.

        """
        return self.created_at < now() - timedelta(seconds=settings.MAX_AGE)



from django.conf import settings as django_settings
from django.core.checks import Error, Warning, register

from . import settings


@register()
def check_settings(app_configs, **kwargs):
    checks = []

    guest_backend = "guest_user.backends.GuestBackend"

    if (
        settings.ENABLED
        and guest_backend not in django_settings.AUTHENTICATION_BACKENDS
    ):
        checks.append(
            Error(
                "The GuestBackend is not in your AUTHENTICATION_BACKENDS. Authenticating guest users will not work.",
                hint='Add "guest_user.backends.GuestBackend" to the AUTHENTICATION_BACKENDS setting.',
                obj="settings",
                id="guest_user.W001",
            )
        )
    elif (
        settings.ENABLED
        and django_settings.AUTHENTICATION_BACKENDS[-1] != guest_backend
    ):
        checks.append(
            Warning(
                "The GuestBackend is not the last item in AUTHENTICATION_BACKENDS. This may have unintended effects.",
                hint='Move the "guest_user.backends.GuestBackend" to the last item in the list.',
                obj="settings",
                id="guest_user.W001",
            )
        )

    return checks



from django.apps import AppConfig


class GuestUserAllauthConfig(AppConfig):
    name = "guest_user.contrib.allauth"
    label = "guest_user_allauth"
    verbose_name = "Guest User Allauth Contrib"

    def ready(self):
        from . import signals  # noqa



import re
from typing import List


class AppSettings:
    """
    This class holds guest user settings.

    Each property corresponds to a setting in the Django project
    prefixed with ``GUEST_USER_``.

    """

    def __init__(self, prefix):
        self.prefix = prefix

    def get(self, name, default):
        from django.conf import settings

        return getattr(settings, self.prefix + name, default)

    @property
    def NAME_GENERATOR(self) -> str:
        """
        Import path to the name generator function for temporary guest usernames.

        Included with the package are three generators:

        - ``guest_user.functions.generate_uuid_username`` (default)

            Generates a UUID4 as a username. Not intended to be shown to the visitor.
            This is the option with the smallest chance for collisions.

        - ``guest_user.functions.generate_numbered_username``

            Will create a random number with ``GUEST_USER_NAME_SUFFIX_DIGITS``
            digits prefixed by ``GUEST_USER_NAME_PREFIX``.

            By default this looks like `Guest4810`, `Guest0588`, etc.

        - ``guest_user.functions.generate_friendly_username``

            Creates a friendly and easy to remember username by combining an adjective, noun and number.
            Requires the ``random_username`` (`PyPI <https://pypi.org/project/random-username/>`_) package.

        If the generator creates a username that already exists, a new one will
        be tried until a unique username has been found.

        """
        return self.get("NAME_GENERATOR", "guest_user.functions.generate_uuid_username")

    @property
    def NAME_PREFIX(self) -> str:
        """
        Random username prefix used with ``generate_numbered_username``.

        :default: ``"Guest"``

        """
        return self.get("NAME_PREFIX", "Guest")

    @property
    def NAME_SUFFIX_DIGITS(self) -> int:
        """
        Number of digits to append to usernames generated by ``generate_numbered_username``.

        :default: ``4``

        """
        return self.get("NAME_SUFFIX_DIGITS", 4)

    @property
    def MAX_AGE(self) -> int:
        """
        Maximum age in seconds for guest sessions to stay valid.

        After this time the session may get deleted by background tasks.

        :default: :ref:`django:ref/settings:``session_cookie_age```

        """
        max_age = self.get("MAX_AGE", None)
        if max_age is None:
            from django.conf import settings as django_settings

            max_age = django_settings.SESSION_COOKIE_AGE
        return max_age

    @property
    def CONVERT_FORM(self) -> str:
        """
        Import path for the form used to convert a guest to a real user.

        The form must have a method ``get_credentials`` to authenticate the user
        after registering.

        :default: :class:`guest_user.forms.UserCreationForm`

        """
        return self.get("CONVERT_FORM", "guest_user.forms.UserCreationForm")

    @property
    def CONVERT_PREFILL_USERNAME(self) -> bool:
        """
        Prefill the generated username in the conversion form.

        :default: ``False``

        """
        return self.get("CONVERT_PREFILL_USERNAME", False)

    @property
    def CONVERT_URL(self) -> str:
        """
        URL name for the convert view.

        :default: ``"guest_user_convert"``

        """
        return self.get("CONVERT_URL", "guest_user_convert")

    @property
    def CONVERT_REDIRECT_URL(self) -> str:
        """
        URL to redirect to after a successful conversion.

        This setting has no effect if a user starts the conversion process from
        a page where they were redirected from with a ``?next=/url/`` query string.

        :default: ``"guest_user_convert_success"``

        """
        return self.get("CONVERT_REDIRECT_URL", "guest_user_convert_success")

    @property
    def REQUIRED_ANON_URL(self) -> str:
        """
        Default redirect target when an anonymous visitor tries to access a view
        where a guest user is required.

        :default: :ref:`django:ref/settings:``login_url```

        """
        url = self.get("REQUIRED_ANON_REDIRECT", None)
        if url is None:
            from django.conf import settings as django_settings

            url = django_settings.LOGIN_URL
        return url

    @property
    def REQUIRED_USER_URL(self) -> str:
        """
        Default redirect target when a registered user tries to access a view
        where a guest user is required.

        :default: :ref:`django:ref/settings:``login_redirect_url```

        """
        url = self.get("REQUIRED_USER_REDIRECT", None)
        if url is None:
            from django.conf import settings as django_settings

            url = django_settings.LOGIN_REDIRECT_URL
        return url

    # return type is the type for the setting, not return type of this property
    @property
    def BLOCKED_USER_AGENTS(self) -> List[str]:
        """
        A list of ignored user agents that will not create guest users.

        Items will be compiled together as a regular expression so you may use regex syntax.

        :default: Googlebot, Mediapartners-Google, Bingbot, Slurp, DuckDuckBot,
                  Baiduspider, Yandex(Mobile)?Bot, Sogou, Exabot, facebot, facebookexternalhit, ia_archiver

        """
        blocked_uas = self.get(
            "BLOCKED_USER_AGENTS",
            [
                "Googlebot",
                "Mediapartners-Google",
                "Bingbot",
                "Slurp",
                "DuckDuckBot",
                "Baiduspider",
                "Yandex(Mobile)?Bot",
                "Sogou",
                "Exabot",
                "facebot",
                "facebookexternalhit",
                "ia_archiver",
            ],
        )

        expression = f"({ ')|('.join(blocked_uas) })"
        return re.compile(expression, re.IGNORECASE)

    @property
    def ENABLED(self) -> bool:
        """
        Disable guest users with this setting.

        :default: ``True``

        .. warning::

           The ``@allow_guest_user`` decorator will **not** enforce
           users to be authenticated if this setting is `False`.

        """
        return self.get("ENABLED", True)

    @property
    def MODEL(self) -> str:
        """
        Specify a different model to use for temporary guest users.

        :default: ``"guest_user.Guest"``

        """
        return self.get("MODEL", "guest_user.Guest")



from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend

from .functions import is_guest_user


class GuestBackend(ModelBackend):
    def authenticate(self, request, username=None, password=None, **kwargs):
        """Authenticate with username only."""

        if password is not None:
            # Prevent authentication when a password was supplied and
            # all previous authentication backends have failed.
            return None

        UserModel = get_user_model()

        try:
            user = UserModel.objects.get(**{UserModel.USERNAME_FIELD: username})
        except UserModel.DoesNotExist:
            return None
        if is_guest_user(user):
            return user
        return None

    def get_user(self, user_id):
        UserModel = get_user_model()
        try:
            user = UserModel._default_manager.get(pk=user_id)
        except UserModel.DoesNotExist:
            return None
        return user



from django.conf import settings as django_settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.shortcuts import redirect

from . import settings
from .functions import is_guest_user, maybe_create_guest_user, redirect_with_next


class AllowGuestUserMixin:
    """
    Allow anonymous users to access the view by creating a guest user.

    This mixin does not require overriding any attributes.

    Example usage:

    .. code:: python

        from guest_user.mixins import AllowGuestUserMixin

        class HelloWorldView(AllowGuestUserMixin, View):
            def get(self, request):
                return HttpResponse(f"Hello {request.user.username}!")

    """

    def dispatch(self, request, *args, **kwargs):
        maybe_create_guest_user(request)
        return super().dispatch(request, *args, **kwargs)


class GuestUserRequiredMixin:
    """
    Current user must be a temporary guest.

    Since being a guest user is not a state that a registered
    user can ever revert back to, there is no "next" URL handling
    in this mixin.

    Example usage:

    .. code:: python

        from guest_user.mixins import GuestUserRequiredMixin

        class OnlyGuestView(GuestUserRequiredMixin, View):
            anonymous_url = "/login/"
            registered_url = "/dashboard/"

    """

    anonymous_url: str = None
    """
    Redirect target for anonymous users.
    Defaults to :attr:`GUEST_USER_REQUIRED_ANON_URL<guest_user.app_settings.AppSettings.REQUIRED_ANON_URL>`.
    """

    registered_url: str = None
    """
    Redirect target for registered users.
    Defaults to :attr:`GUEST_USER_REQUIRED_USER_URL<guest_user.app_settings.AppSettings.REQUIRED_USER_URL>`.
    """

    def dispatch(self, request, *args, **kwargs):
        if is_guest_user(request.user):
            return super().dispatch(request, *args, **kwargs)
        if request.user.is_anonymous:
            redirect_url = self.anonymous_url or settings.REQUIRED_ANON_URL
        else:
            redirect_url = self.registered_url or settings.REQUIRED_USER_URL
        return redirect(redirect_url)


class RegularUserRequiredMixin:
    """
    Current user must not be a temporary guest.

    Anonymous users will be redirected to `login_url`.
    Guest users will be redirected to the `convert_url`.

    The redirected URL will get a "next" parameter added to the URL
    in order to redirect the user back to the page they were trying to access.

    Example usage:

    .. code:: python

        from guest_user.mixins import RegularUserRequiredMixin

        class RealUsersOnlyView(RegularUserRequiredMixin, View):
            login_url = "/login/"
            convert_url = "/convert/?from=RealUsersOnlyView"

    """

    login_url: str = None
    """
    Redirect target for anonymous users.
    Defaults to :ref:`django:ref/settings:``login_url```.
    """

    convert_url: str = None
    """
    Redirect target for guest users.
    Defaults to :attr:`GUEST_USER_CONVERT_URL<guest_user.app_settings.AppSettings.CONVERT_URL>`.
    """

    redirect_field_name: str = REDIRECT_FIELD_NAME
    """
    URL parameter used to redirect to the origin page. Defaults to
    :attr:`django.contrib.auth.REDIRECT_FIELD_NAME<django:django.contrib.auth.mixins.AccessMixin.redirect_field_name>`
    (= "next").
    """

    def get_login_url(self):
        if not self.request.user.is_anonymous:
            return self.convert_url or settings.CONVERT_URL
        return super().get_login_url()

    def dispatch(self, request, *args, **kwargs):
        user = request.user
        if user.is_authenticated and not is_guest_user(user):
            return super().dispatch(request, *args, **kwargs)
        if user.is_anonymous:
            redirect_url = self.login_url or django_settings.LOGIN_URL
        else:
            redirect_url = self.convert_url or settings.CONVERT_URL
        return redirect_with_next(request, redirect_url, self.redirect_field_name)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['guest_user',
 'guest_user.contrib',
 'guest_user.contrib.allauth',
 'guest_user.contrib.tos',
 'guest_user.management',
 'guest_user.management.commands',
 'guest_user.migrations',
 'guest_user.templatetags']

package_data = \
{'': ['*'], 'guest_user': ['templates/guest_user/*']}

setup_kwargs = {
    'name': 'django-guest-user',
    'version': '0.5.4',
    'description': 'A Django app that lets visitors interact with your site without registration.',
    'long_description': '[![Code Lint](https://github.com/julianwachholz/django-guest-user/actions/workflows/lint.yml/badge.svg)](https://github.com/julianwachholz/django-guest-user/actions/workflows/lint.yml)\n[![Python Tests](https://github.com/julianwachholz/django-guest-user/actions/workflows/test.yml/badge.svg)](https://github.com/julianwachholz/django-guest-user/actions/workflows/test.yml)\n[![Documentation](https://readthedocs.org/projects/django-guest-user/badge/?style=flat)](https://django-guest-user.readthedocs.io)\n\n# django-guest-user\n\nAllow visitors to interact with your site like a temporary user ("guest")\nwithout requiring registration.\n\nAnonymous visitors who request a decorated page get a real temporary user object\nassigned and are logged in automatically. They can use the site like a normal\nuser until they decide to convert to a real user account to save their data.\n\nInspired by and as an alternative for [django-lazysignup](https://github.com/danfairs/django-lazysignup)\nand rewritten for Django 3.2+ and Python 3.7+.\n\n## Documentation\n\nFind the [**complete documentation**](https://django-guest-user.readthedocs.io/)\non Read the Docs.\n\n## Quickstart\n\n1. Install the `django-guest-user` package from PyPI\n2. Add `guest_user` to your `INSTALLED_APPS` and migrate your database\n3. Add `guest_user.backends.GuestBackend` to your `AUTHENTICATION_BACKENDS`\n4. Include `guest_user.urls` in your URLs\n5. Decorate your views with `@allow_guest_user`:\n\n   ```python\n   from guest_user.decorators import allow_guest_user\n\n   @allow_guest_user\n   def my_view(request):\n       assert request.user.is_authenticated\n       return render(request, "my_view.html")\n   ```\n\nA more detailed guide is available in the\n[installation documentation](https://django-guest-user.readthedocs.io/en/latest/setup.html#how-to-install).\n\n## Contributing\n\nAll contributions are welcome! Please read the\n[contributing guidelines](CONTRIBUTING.md) in this repostory.\n\n## Development Status\n\nThis project is under active development. Thanks to\n[previous work](https://github.com/danfairs/django-lazysignup) the core\nfunctionality is well-established and this package builds on top of it.\n\nThis project was created because the original project has been in an inactive\nstate without major updates in a long time. The code base was rewritten with\nonly modern versions of Python and Django in mind.\n',
    'author': 'Julian Wachholz',
    'author_email': 'julian@wachholz.ch',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/julianwachholz/django-guest-user',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# Generated by Django 3.2.8 on 2021-10-23 16:13

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Guest",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, db_index=True, verbose_name="Created at"
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="guest+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "verbose_name": "Guest",
                "verbose_name_plural": "Guests",
                "ordering": ["-created_at"],
                "swappable": "GUEST_USER_MODEL",
            },
        ),
    ]



from django.core.management.base import BaseCommand

from ...functions import get_guest_model


class Command(BaseCommand):
    help = "Delete expired guest users."

    def handle(self, **options):
        """Delete every user"""
        GuestModel = get_guest_model()
        GuestModel.objects.delete_expired()
